#ifndef DUNE_CURVEDGRID_GRIDFUNCTION_HH
#define DUNE_CURVEDGRID_GRIDFUNCTION_HH

#include <type_traits>

#include <dune/common/typeutilities.hh>
#include <dune/curvedgrid/concepts.hh>
#include <dune/curvedgrid/gridfunctions/analyticgridfunction.hh>

namespace Dune {
namespace Impl {

struct GridOf
{
private:
  template <class ES, class Grid = typename ES::Grid>
  static auto typeImpl(ES const&, PriorityTag<3>) -> Grid;

  template <class ES, class Grid = typename ES::GridView::Grid>
  static auto typeImpl(ES const&, PriorityTag<2>) -> Grid;

  template <class ES>
  static auto typeImpl(ES const& es, PriorityTag<1>) -> decltype(es.grid());

  template <class ES>
  static auto typeImpl(ES const& es, PriorityTag<0>) -> decltype(es.gridView().grid());

public:
  template <class GF>
  static auto type(GF const& gf)
    -> decltype(typeImpl(gf.entitySet(), PriorityTag<4>{}));
};

} // end namespace Impl


/// \brief Type-Traits to extract the grid-type from a GridFunction `GF`
template <class GF>
using GridOf_t = decltype( Impl::GridOf::type(std::declval<GF>()) );


/// \brief Conditionally define the GridFunction type.
/**
 * If the type `GF` is a GridFunction on the grid `Grid`, use this type as GridFunction type,
 * Otherwise it is a functor and construct an \ref AnalyticGridFunction on the `Grid`.
 **/
template <class Grid, class GF>
using GridFunctionOf_t
  = std::conditional_t<Concept::isGridFunction<GF, Grid>(), GF, AnalyticGridFunction<Grid,GF>>;

} // end namespace Dune

#endif // DUNE_CURVEDGRID_GRIDFUNCTION_HH
